@@ -13,7 +13,9 @@ from flask import (
 from flask_caching import Cache
 
 from flask_compress import Compress, DictCache
-from flask_compress.flask_compress import _choose_algorithm
+from flask_compress.flask_compress import _choose_algorithm, _uncompress_data
+
+ALGORITHMS = ("gzip", "deflate", "br", "zstd")
 
 
 class DefaultsTest(unittest.TestCase):
@@ -134,11 +136,11 @@ class UrlTests(unittest.TestCase):
         self.app = Flask(__name__)
         self.app.testing = True
 
-        small_path = os.path.join(os.getcwd(), "tests", "templates", "small.html")
-        large_path = os.path.join(os.getcwd(), "tests", "templates", "large.html")
+        self.small_path = os.path.join(os.getcwd(), "tests", "templates", "small.html")
+        self.large_path = os.path.join(os.getcwd(), "tests", "templates", "large.html")
 
-        self.small_size = os.path.getsize(small_path) - 1
-        self.large_size = os.path.getsize(large_path) - 1
+        self.small_size = os.path.getsize(self.small_path) - 1
+        self.large_size = os.path.getsize(self.large_path) - 1
 
         Compress(self.app)
 
@@ -150,6 +152,20 @@ class UrlTests(unittest.TestCase):
         def large():
             return render_template("large.html")
 
+    def test_compressed_content(self):
+        client = self.app.test_client()
+        with open(self.large_path, "rb") as f:
+            original_data = f.read().rstrip()  # flask strips trailing newline
+
+        for algorithm in ALGORITHMS:
+            headers = [("Accept-Encoding", algorithm)]
+            response = client.get("/large/", headers=headers)
+            self.assertIn("Content-Encoding", response.headers)
+            self.assertEqual(response.status_code, 200)
+            self.assertEqual(response.is_streamed, True)
+            self.assertGreater(self.large_size, len(response.data))
+            self.assertEqual(original_data, _uncompress_data(response.data, algorithm))
+
     def client_get(self, ufs):
         client = self.app.test_client()
         response = client.get(ufs, headers=[("Accept-Encoding", "gzip")])
@@ -462,6 +478,13 @@ class StreamTests(unittest.TestCase):
         self.app = Flask(__name__)
         self.app.testing = True
 
+        self.app.config["COMPRESS_ALGORITHM_STREAMING"] = [
+            "zstd",
+            "br",
+            "gzip",  # not by default for streaming
+            "deflate",
+        ]
+
         self.file_path = os.path.join(os.getcwd(), "tests", "templates", "large.html")
         self.file_size = os.path.getsize(self.file_path)
 
@@ -478,7 +501,8 @@ class StreamTests(unittest.TestCase):
         Compress(self.app)
         self.app.config["COMPRESS_STREAMS"] = False
         client = self.app.test_client()
-        for algorithm in ("gzip", "deflate", "br", "zstd", ""):
+
+        for algorithm in (*ALGORITHMS, ""):
             headers = [("Accept-Encoding", algorithm)]
             response = client.get("/stream/large", headers=headers)
             self.assertEqual(response.status_code, 200)
@@ -488,31 +512,37 @@ class StreamTests(unittest.TestCase):
     def test_compression_stream(self):
         Compress(self.app)
         client = self.app.test_client()
-        for algorithm in ("deflate", "br", "zstd"):
+        with open(self.file_path, "rb") as f:
+            original_data = f.read()
+
+        for algorithm in ALGORITHMS:
             headers = [("Accept-Encoding", algorithm)]
             response = client.get("/stream/large", headers=headers)
             self.assertIn("Content-Encoding", response.headers)
             self.assertEqual(response.status_code, 200)
             self.assertEqual(response.is_streamed, True)
             self.assertGreater(self.file_size, len(response.data))
+            self.assertEqual(original_data, _uncompress_data(response.data, algorithm))
 
-        # gzip is not supported for streamed responses
-        headers = [("Accept-Encoding", "gzip")]
+        headers = [("Accept-Encoding", "")]
         response = client.get("/stream/large", headers=headers)
         self.assertNotIn("Content-Encoding", response.headers)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.is_streamed, True)
         self.assertEqual(self.file_size, len(response.data))
+        self.assertEqual(original_data, response.data)
 
 
 class StreamTestsWithETags(unittest.TestCase):
     def setUp(self):
-        self.app = Flask(__name__)
+        self.app = Flask(__name__, static_folder="web", static_url_path="/path")
         self.app.testing = True
 
         self.file_path = os.path.join(os.getcwd(), "tests", "templates", "large.html")
         self.file_size = os.path.getsize(self.file_path)
 
+        self.app.config["COMPRESS_MIN_SIZE"] = 1
+
         Compress(self.app)
 
         @self.app.route("/stream/large")
@@ -528,10 +558,13 @@ class StreamTestsWithETags(unittest.TestCase):
 
     def test_conditionals_are_skipped_on_streaming(self):
         client = self.app.test_client()
+
         r1 = client.get("/stream/large", headers=[("Accept-Encoding", "br")])
         tag, is_weak = r1.get_etag()
         self.assertEqual(r1.status_code, 200)
         self.assertEqual(r1.is_streamed, True)
+        self.assertIn("Content-Encoding", r1.headers)
+        self.assertEqual(r1.headers.get("Content-Encoding"), "br")
         self.assertEqual(tag, "stream-etag:br")
         self.assertFalse(is_weak)
         r1.close()
@@ -542,6 +575,39 @@ class StreamTestsWithETags(unittest.TestCase):
         )
         self.assertEqual(r2.status_code, 200)  # Should be 200, not 304
         self.assertEqual(r2.is_streamed, True)
+        self.assertIn("Content-Encoding", r1.headers)
+        self.assertEqual(r1.headers.get("Content-Encoding"), "br")
+        r2.close()
+
+    def test_static_exception_to_conditionals(self):
+        # Here we test that the static endpoint, which is using streaming responses,
+        # still respects conditional requests even when streaming.
+        # We use a custom static folder and static url path to show that the test
+        # works even if those are changed from the defaults.
+        # In practice there is almost no chance for a user to accidentally write a
+        # streaming 'static' endpoint, as this will raise an exception in Flask,
+        # because of the predefined static file handling.
+        client = self.app.test_client()
+
+        r1 = client.get("/path/test.xml", headers=[("Accept-Encoding", "br")])
+        tag, is_weak = r1.get_etag()
+        self.assertEqual(r1.status_code, 200)
+        self.assertEqual(r1.is_streamed, True)
+        self.assertIn("Content-Encoding", r1.headers)
+        self.assertEqual(r1.headers.get("Content-Encoding"), "br")
+        self.assertIsNotNone(tag)
+        self.assertEqual(tag[-3:], ":br")
+        self.assertFalse(is_weak)
+        r1.close()
+
+        r2 = client.get(
+            "/path/test.xml",
+            headers=[("Accept-Encoding", "br"), ("If-None-Match", r1.headers["ETag"])],
+        )
+        self.assertEqual(r2.status_code, 304)  # static endpoint has an exception
+        self.assertEqual(r2.is_streamed, True)
+        self.assertIn("Content-Encoding", r1.headers)
+        self.assertEqual(r1.headers.get("Content-Encoding"), "br")
         r2.close()
 
 
